﻿Public Class DataProviderFactory
  ' Methods
  Shared Sub New()
    DataProviderFactory._providerInfos = New List(Of ProviderInfo)
  End Sub

  ''' <summary>
  ''' Deinitializes the factory. Used for unit testing only.
  ''' </summary>
  Public Shared Sub Deinitialize()
    DataProviderFactory._providerInfos.Clear()
  End Sub

  Public Shared Function GetProvider(ByVal providerName As String) As IDataProvider
    Return DataProviderFactory._providerInfos.Where(Of ProviderInfo)(Function (ByVal info As ProviderInfo) 
    Return (info.Name = providerName)
    End Function).First(Of ProviderInfo)().Singleton
  End Function

  Public Shared Sub RegisterProvider(ByVal providerName As String, ByVal creator As Func(Of IDataProvider))
    Dim pi As New ProviderInfo
    pi.Name = providerName
    pi.Creator = creator
    DataProviderFactory._providerInfos.Add(pi)
  End Sub

  Public Shared Sub RegisterProvider(ByVal providerName As String, ByVal provider As Type, ByVal parameter As String)
    Dim pi As New ProviderInfo
    pi.Name = providerName
    pi.Type = provider
    pi.Parameter = parameter
    DataProviderFactory._providerInfos.Add(pi)
  End Sub

  ' Fields
  Private Shared ReadOnly _providerInfos As List(Of ProviderInfo)

  ' Nested Types
  Private Class ProviderInfo
    ' Methods
    Public Sub New()

      ' Properties
    Public Property Creator() As Func(Of IDataProvider)
      Get
        Return _Creator
      End Get
      Set(ByVal value As Func(Of IDataProvider))
        _Creator = value
      End Set
    End Property
    Private _Creator As Func(Of IDataProvider)

    Public Property Name() As String
      Get
        Return _Name
      End Get
      Set(ByVal value As String)
        _Name = value
      End Set
    End Property
    Private _Name() As String

    Public Property Parameter() As String
      Get
        Return _Parameter
      End Get
      Set(ByVal value As String)
        _Parameter = value
      End Set
    End Property
    Private _Parameter As String

    Public ReadOnly Property Singleton() As IDataProvider
      Get
        Return _Singleton
      End Get
    End Property
    Private _Singleton As IDataProvider

    Public Property Type() As Type
      Get
        Return _Type
      End Get
      Set(ByVal value As Type)
        _Type = value
      End Set
    End Property
    Private _Type As Type
  End Class
End Class
