﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Net;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Frmwrk.ComponentModel;
using Lobs.ApplicationModel;
using Lobs.Views;

namespace Lobs.ViewModel
{
  public class EmployeesViewModel : ViewModelBase
  {
    internal UIElement CreateView()
    {
      View = new EmployeesView();
      View.DataContext = this;
      return View;
    }

    EmployeesView View { get; set; }

    /// <summary>
    /// Loads the employees synchron by freezing the UI. Works not in Silverlight!
    /// </summary>
    internal void LoadEmployeesSynchron()
    {
      // Load employees synchronously
      var employees = ApplicationProcess.LoadEmployees();
      
      foreach (var employee in employees)
      {
        // Load photo of each employee
        employee.LoadPhoto();

        // Convert employee to view model
        _employees.Add(Employee.FromApplicationModel(employee));
      }
    }

    internal void LoadEmployeesAsynchron()
    {
      var service = new AsyncServiceFunction<List<ApplicationModel.Employee>>(ApplicationProcess.LoadEmployees);
      service.ServiceCompleted += (sender, e) =>
      {
        var list = new List<Employee>();

        foreach (var item in e.Result)
        {
          Employee employee = Employee.FromApplicationModel(item);
          _employees.Add(employee);
          list.Add(employee);
        }
        // Load photos in a background thread
        ThreadPool.QueueUserWorkItem(obj => LoadEmployeePhotos(list), null);
      };
      service.InvokeServiceAsync();
    }

    /// <summary>
    /// Loads the employee photos in a background thread.
    /// </summary>
    void LoadEmployeePhotos(object obj)
    {
      // Must not use the ObservableCollection Employees here, it belongs to the UI thread
      var list = (List<Employee>)obj;
      foreach (var item in list)
      {
        if (item.Photo == null)
        {
          // Create local variable for the closure used in the lambda expression
          Employee employee = item;

          // Load photo to application model employee
          employee.AppEmployee.LoadPhoto();

          // Copy photo from application to view model - this must be done in UI thread
          SendToView(() => { employee.PhotoBytes = employee.AppEmployee.Photo; });

          // Slow down the process of loading the photos to make it better watchable
          Thread.Sleep(750);
        }
      }
    }

    public ObservableCollection<Employee> Employees
    {
      get { return _employees; }
    }

    readonly ObservableCollection<Employee> _employees = new ObservableCollection<Employee>();
  }
}
