﻿Imports System.Collections.ObjectModel
Imports System.Windows
Imports System.Threading
Imports Frmwrk.ComponentModel
Imports Lobs.Views

Namespace Lobs.ViewModel
  Public Class EmployeesViewModel
    Inherits ViewModelBase
    ' Methods
    Public Sub New()
      Me._employees = New ObservableCollection(Of Employee)
    End Sub

    Friend Function CreateView() As UIElement
      Me.View = New Lobs.Views.EmployeesView
      Me.View.DataContext = Me
      Return Me.View
    End Function

    Friend Sub LoadEmployeesSynchron()
      ' Load employees synchronously
      Dim employees As List(Of ApplicationModel.Employee) = MyBase.ApplicationProcess.LoadEmployees
      Dim employee1 As ApplicationModel.Employee
      For Each employee1 In employees
        ' Load photo of each employee
        employee1.LoadPhoto()
        ' Convert employee to view model
        Me._employees.Add(Employee.FromApplicationModel(employee1))
      Next
    End Sub

    Friend Sub LoadEmployeesAsynchron()
      Dim service As New AsyncServiceFunction(Of List(Of ApplicationModel.Employee))(New Func(Of List(Of ApplicationModel.Employee))(AddressOf MyBase.ApplicationProcess.LoadEmployees))
      AddHandler service.ServiceCompleted, AddressOf LoadEmployeesCompleted
      service.InvokeServiceAsync()
    End Sub

    Function LoadEmployeesCompleted(ByVal sender As Object, ByVal e As AsyncServiceCompletedEventArgs(Of List(Of ApplicationModel.Employee)))
      Dim list As New List(Of Employee)
      Dim item As ApplicationModel.Employee
      For Each item In e.Result
        Dim employee As Employee = employee.FromApplicationModel(item)
        Me._employees.Add(employee)
        list.Add(employee)
      Next
      ThreadPool.QueueUserWorkItem(Function(obj As Object) Me.LoadEmployeePhotos(list), Nothing)
      Return True
    End Function

    Private Function LoadEmployeePhotos(ByVal obj As Object) As Boolean
      Dim list As List(Of Employee) = DirectCast(obj, List(Of Employee))
      Dim item As Employee
      For Each item In list
        If (item.Photo Is Nothing) Then
          Dim employee As Employee = item
          employee.AppEmployee.LoadPhoto()
          employeeToBeUpdated = employee
          MyBase.SendToView(AddressOf UpdatePhotoSub)
          Thread.Sleep(750)
        End If
      Next
    End Function

    Dim employeeToBeUpdated As Employee
    Sub UpdatePhotoSub()
      UpdatePhoto(employeeToBeUpdated)
    End Sub

    Sub UpdatePhoto(ByVal employee As Employee)
      employee.PhotoBytes = employee.AppEmployee.Photo
    End Sub

    Public ReadOnly Property Employees() As ObservableCollection(Of Employee)
      Get
        Return Me._employees
      End Get
    End Property

    Private Property View() As EmployeesView
      Get
        Return _view
      End Get
      Set(ByVal value As EmployeesView)
        _view = value
      End Set
    End Property
    Private _view As EmployeesView

    Private ReadOnly _employees As ObservableCollection(Of Employee)
  End Class
End Namespace