﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Windows.Controls;
using System.Windows.Markup;

namespace Frmwrk.Windows.Controls.Basis
{
  /// <summary>
  /// Base class for all user controls.
  /// </summary>
  public class UserControlBase : UserControl, INotifyPropertyChanged // @tag#1
  {
    /// <summary>
    /// Raises the property changed event for the specified property expression if PropertyChanged event is defined.
    /// </summary>
    protected void RaisePropertyChanged<TResult>(Expression<Func<TResult>> propertyExpression)
    {
      RaisePropertyChanged(((MemberExpression)propertyExpression.Body).Member.Name);
    }

    /// <summary>
    /// Raises the property changed event for the specified property name if PropertyChanged event is defined.
    /// </summary>
    protected void RaisePropertyChanged(string propertyName)
    {
      if (PropertyChanged != null)
        PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
    }

    /// <summary>
    /// Property changed event for interface INotifyPropertyChanged.
    /// </summary>
    public event PropertyChangedEventHandler PropertyChanged;
  }
}
